/**
 * 
 */
package eu.etaxonomy.taxeditor.navigation.navigator.handler;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.ChangeAcceptedTaxonToSynonymOperation;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.DeleteOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 * <p>ChangeAcceptedTaxonToSynonymHandler class.</p>
 *
 * @author n.hoffmann
 * @created Jan 4, 2010
 * @version 1.0
 */
public class ChangeAcceptedTaxonToSynonymHandler extends DeleteHandler
		implements IHandler, IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(ChangeAcceptedTaxonToSynonymHandler.class);
	private ChangeAcceptedTaxonToSynonymOperation operation;

	private UUID newAcceptedTaxonNodeUuid;
	private TaxonNameEditor editor;
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		
		 activePage = HandlerUtil.getActiveWorkbenchWindow(event).getActivePage();

		taxonNavigator = NavigationUtil.showNavigator();

		TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);

		String plural = selection.size() > 1 ? "s" : "";
		// Prompt user for confirmation
		
		

		Iterator selectionIterator = selection.iterator();
		Set<ITaxonTreeNode> treeNodes = new HashSet<ITaxonTreeNode>();

		while (selectionIterator.hasNext()){
			Object object = selectionIterator.next();
			if(object instanceof ITaxonTreeNode) {
                treeNodes.add((ITaxonTreeNode) object);
            }
		}
		boolean allEditorsClosed = true;
		for (ITaxonTreeNode treeNode : treeNodes){
			if(treeNode instanceof TaxonNode) {
				allEditorsClosed &= closeObsoleteEditor((TaxonNode) treeNode);
			}
		}
		AbstractPostOperation operation = null;
		if (treeNodes.size() == 1 ){
			try {
				
				ITaxonTreeNode treeNode = treeNodes.iterator().next();
				ITaxonTreeNode oldAcceptedTaxonNode =treeNode;
				
				// check if taxon has no children
				if(((TaxonNode)oldAcceptedTaxonNode).getCountChildren() > 0) {
					MessagingUtils.warningDialog("Prerequisite not met", this, 
							"The accepted taxon must not have any childen. You need to move all childen to " +
							"another taxon node in the TaxonNavigator before attempting to turn the accepted " +
							"taxon into a synonym.");
					return null;
				}
				/*if(((TaxonNode)oldAcceptedTaxonNode).getTaxon().hasSynonyms()) {
					EditorUtil.warningDialog("Prerequisite not met", this, 
							"The accepted taxon must not have any synonyms. You need to move all synonyms to " +
							"another taxon before attempting to turn the accepted " +
							"taxon into a synonym.");
					return null;
				}*/
				
				List<UUID> excludeTaxa = new ArrayList<UUID>();
				excludeTaxa.add(oldAcceptedTaxonNode.getUuid());
				TaxonNode newAcceptedTaxonNode = TaxonNodeSelectionDialog.select(HandlerUtil.getActiveShell(event), 
						taxonNavigator.getConversationHolder(), 
						"Choose the accepted taxon", 
						excludeTaxa, 
						null, 
						((TaxonNode)oldAcceptedTaxonNode).getClassification());			

				if (newAcceptedTaxonNode == null) {
					return null;
				}
	
				if (allEditorsClosed){
					
						operation = new ChangeAcceptedTaxonToSynonymOperation(event.getCommand().getName(), NavigationUtil.getUndoContext(), oldAcceptedTaxonNode, newAcceptedTaxonNode,
								taxonNavigator, taxonNavigator);
			
						NavigationUtil.executeOperation(operation);
						//}
				}
	
		
	
			} catch (NotDefinedException e) {
				MessagingUtils.warn(getClass(), "Command name not set");
			}
		} else{
			if( MessageDialog.openConfirm(HandlerUtil.getActiveShell(event), "Moving to synonymy", "The operation move accepted taxon to synonymy is available only for a single taxon.")){
				return null;
			}
		}
		
		
		return null;
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	/** {@inheritDoc} */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		Display.getDefault().asyncExec(new Runnable(){

			public void run() {
				EditorUtil.close(editor.getMultiPageTaxonEditor());
				
				try {
					MultiPageTaxonEditor possibleOpenEditor = (MultiPageTaxonEditor) EditorUtil.findEditorByTaxonNodeUuid(newAcceptedTaxonNodeUuid);
					if(possibleOpenEditor != null){
						EditorUtil.close(possibleOpenEditor);
					}
					EditorUtil.openTaxonNode(newAcceptedTaxonNodeUuid);
				} catch (PartInitException e) {
					MessagingUtils.error(this.getClass(), e);
					throw new RuntimeException(e);
				} catch (Exception e) {
					MessagingUtils.warningDialog("Could not create Taxon", this, e.getMessage());
				}
			}
			
		});
		
		
		return true;
	}

	/**
	 * <p>onComplete</p>
	 *
	 * @return a boolean.
	 */
	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}	
}
